CCamera = {};

function CCamera:new() 
--
	local object = 
	{
		left = 0,
		right = 320,
		top = 0,
		bottom = 180,
		w = 320,
		h = 180,
		
		left_bound = 0,
		right_bound = 320,
		top_bound = 0,
		bottom_bound = 180
	};
	
	setmetatable( object, { __index = CCamera } );
	
	return object;
--
end;	

function CCamera:SetSize(w, h)
--
	self.w = w;
	self.h = h;

	self.right = self.left + w;
	self.bottom = self.top + h;
--
end;

function CCamera:SetBounds(x1, y1, x2, y2)
--
	self.left_bound = x1;
	self.right_bound = x2;
	self.top_bound = y1;
	self.bottom_bound = y2;
--
end;

function CCamera:SetPos(x, y)
--
	self.left = x;
	self.right = x + self.w;
	
	self.top = y;
	self.bottom = y + self.h;
	
	self:KeepInBounds();
--
end;

function CCamera:Move(dist_x, dist_y)
--
	self.left = self.left + dist_x;
	self.right = self.left + self.w;
	
	self.top = self.top + dist_y;
	self.bottom = self.top + self.h;
	
	self:KeepInBounds();
--
end;

function CCamera:KeepInBounds()
--
	if( self.left < self.left_bound )then
		self.left = self.left_bound;
		self.right = self.left + self.w;
	end;
	
	if( self.top < self.top_bound )then
		self.top = self.top_bound;
		self.bottom = self.top + self.h;
	end;
	
	if( self.right > self.right_bound )then
		self.left = self.right_bound - self.w;
		self.right = self.left + self.w;
	end;
	
	if( self.bottom > self.bottom_bound )then
		self.top = self.bottom_bound - self.h;
		self.bottom = self.top + self.h;
	end;
	
	math.floor(self.left);
	math.floor(self.right);
	math.floor(self.top);
	math.floor(self.bottom);
--
end;

function CCamera:WorldToScreenX(x)
--
	return x - self.left;
--
end;

function CCamera:WorldToScreenY(y)
--
	return y - self.top;
--
end;
