require "camera"
require "player"
require "point"
require "rect"

MAX_PRESSED_TIME = 0.08;
TIMESTEP = 1 / 60;

-- going clockwise
CP_RIGHT = 1;
CP_BOTTOM_LEFT = 2;
CP_BOTTOM_RIGHT = 3;
CP_LEFT = 4;
CP_TOP_LEFT = 5;
CP_TOP_RIGHT = 6;

gravity = 1500;
scale = 2; -- Adjust zoom with this

--canvas = love.graphics.newCanvas(320,180)
player = CPlayer:new();
cam = CCamera:new();

move_left_pressed = false;
move_right_pressed = false;

jump_pressed = false;
do_jump = false;
pressed_time = 0;

accumulator = 0;

loader = require("ATL.Loader");
loader.path = "maps/" --Change this to wherever your .tmx files are
map = loader.load("tilemap.tmx") --Change this to the name of your mapfile
map.useSpriteBatch = true;
collision_layer = map.layers[ "CollLayer" ];
slopeHeights = {};

function tileIsFlipped(layer, direction, x, y)
	local flippedTile = layer._flippedTiles(x,y);
	if ( flippedTile ) then
		if direction == "diagonal" then
			return flippedTile % 2 == 1
		elseif direction == "vertical" then
			return flippedTile % 4 >= 2
		elseif direction == "horizontal" then
			return flippedTile >= 4
		end;
	else
		return false;
	end;
end;

function IsMapLayerTileFlippedHorz( mapLayer, x, y )
--
	local flipped = false;
	local flipX = nil;
	local flippedTile = mapLayer._flippedTiles(x,y);
	if ( flippedTile ) then
		flipX = flippedTile >= 4 and -1 or 1;
		if ( flipX < 0 ) then flipped = true; else flipped = false; end;
	end;
	return( flipped );
--
end;

function IsMapLayerTileFlippedVert( mapLayer, x, y )
--
	local flipped = false;
	local flipY = nil;
	local flippedTile = mapLayer._flippedTiles( x, y );
	if ( flippedTile ) then
		flipY = (flippedTile % 4) >= 2 and -1 or 1;
		if ( flipY < 0 ) then flipped = true; else flipped = false; end;
	end;
	return( flipped );
--
end;

-- Convert from CSV string to table
-- from the CSV utils page on the lua wiki
-- http://lua-users.org/wiki/CsvUtils
function fromCSV (s)
--
	s = s .. ',' ;            -- ending comma
	local t = {};           -- table to collect fields
	local fieldstart = 1;

	repeat
	--
		-- next field is quoted? (start with `"'?)
		if ( string.find( s, '^"', fieldstart ) ) then
		--
			local a, c;
			local i  = fieldstart;
			
			repeat
				-- find closing quote
				a, i, c = string.find( s, '"("?)', i+1 );
			until ( c ~= '"' );    -- quote not followed by quote?
			
			if ( not( i  ) ) then 
				error('unmatched "'); 
			end;
			
			local f = string.sub( s, fieldstart+1, i-1 );
			table.insert( t, ( string.gsub( f, '""', '"' ) ) );
			fieldstart = string.find( s, ',', i ) + 1;
		--
		else                -- unquoted; find next comma
		--
			local nexti = string.find( s, ',', fieldstart );
			table.insert( t, string.sub( s, fieldstart, nexti-1 ) );
			fieldstart = nexti + 1;
		--	
		end;
	--	
	until ( fieldstart > string.len(s) );

	return( t );
--
end

function CreateMergedRect(rect1, rect2)
--
	local mergedRect = { left = 0, right = 0, top = 0, bottom = 0 };
		
	mergedRect.left = math.min(rect1.left, rect2.left);
	mergedRect.right = math.max(rect1.right, rect2.right);
	mergedRect.top = math.min(rect1.top, rect2.top);
	mergedRect.bottom = math.max(rect1.bottom, rect2.bottom);
	
	return mergedRect;
--
end

function round(number)
  floorNumber = math.floor(number)
  decimalNumber = number - floorNumber

  if (decimalNumber >= .5) then
    return(math.ceil(number))
  end

  if (decimalNumber < .5) then
    return(math.floor(number))
  end
end

function ModifiedFloor( number )
--        
	--Number to round up at.
	roundingNumber = .9999999

	floorNumber = math.floor( number );
	decimalNumber = number - floorNumber;

	if( decimalNumber >= roundingNumber )then
		return( math.ceil( number ) );
	else
		return( math.floor( number ) );
	end;
--
end;

function floor( val )
--
	--return( math.floor( val ) );
	return( ModifiedFloor( val ) );
--
end;